/*
 * benchmark program:   complex_update2.c
 * 
 * benchmark suite:     DSP-kernel
 * 
 * description:         complex_update - filter benchmarking
 * 
 * This program performs a complex update of the form D = C + A*B,
 * where A, B, C and D are complex numbers .
 * 
 *          A = Ar + j Ai
 *          B = Br + j Bi
 *          C = Cr + j Ci
 *          D = C + A*B 
 *                      =>  Dr = Cr + Ar*Br - Ai*Bi
 *                      =>  Di = Ci + Ar*Bi + Ai*Br
 * 
 * reference code:      target assembly
 * 
 * f. verification:     simulator based
 * 
 *  organization:        Aachen University of Technology - IS2 
 *                       DSP Tools Group
 *                       phone:  +49(241)807887 
 *                       fax:    +49(241)8888195
 *                       e-mail: zivojnov@ert.rwth-aachen.de 
 *
 * author:              Juan Martinez Velarde
 * 
 * history:             25-5-94 creation (Martinez Velarde)
 * 
 */

#define STORAGE_CLASS register
#define TYPE  int

void
pin_down(TYPE *dr, TYPE *di)
{
   
  *dr = 0 ; 
  *di = 0 ; 

}


TYPE
main()
{
  static TYPE ar = 2, ai = 1 ;
  static TYPE br = 2, bi = 5 ; 
  static TYPE cr = 3, ci = 4 ; 
  static TYPE dr = 0, di = 0 ; 
  
  pin_down(&dr, &di) ; 
  
  __asm("START_PROFILING:") ; 
  
  dr  = cr + ar * br ;
  dr -=      ai * bi ; 

  di  = ci + ai * br ; 
  di +=      ar * bi ; 
   
  __asm("END_PROFILING:") ; 
  
  printf("dr = %i\n",dr) ;
  printf("di = %i\n",di) ;

  pin_down(&dr, &di) ; 
  
  return(0)  ; 
}
