/*
 * benchmark program:   dot_product_ext.c
 * 
 * benchmark suite:     DSP-kernel
 * 
 * description:         dot product benchmarking
 *
 * This program performs a dot product of the form Z=AB,
 * where A is a [1x2] vector and B is a [2x1] vector.
 * 
 *          A[1 x 2] * B[2 x 1] = Z
 *                  
 * vector A[1 x 2]= |a1 a2|
 *
 * vector B[2 x 1]= | b1 |
 *                  | b2 |
 *
 * dot product Z = a1*b1 + a2* b2
 * 
 * vector elements are stored as
 *
 * A[1 x 2] = { a1, a1 }
 * 
 * B[2 x 1] = { b1, b2 }
 * 
 * 
 * reference code:       none
 * 
 * f. verification:      with printf function
 * 
 * organization:         Aachen University of Technology - IS2 
 *                       DSP Tools Group
 *                       phone:  +49(241)807887 
 *                       fax:    +49(241)8888195
 *                       e-mail: zivojnov@ert.rwth-aachen.de 
 *
 * author:              Juan Martinez Velarde
 * 
 * history:             10-05-94 C Code creation (Martinez Velarde)(dot_product.c)
 *                      05-03-95 language extensions (Harald L. Schraut)
 *
 *                      $Author: schraut $
 *                      $Date: 1995/03/05 14:35:31 $
 *                      $Revision: 1.1 $
 */

#define STORAGE_CLASS  register
#define TYPE           int

void pin_down(TYPE dm *Z)
{
  *Z = 0 ; 
}

TYPE main()
{ 
  static  TYPE pm A[2] = {2,1} ; 
  static  TYPE dm B[2] = {2,5} ;
  static  TYPE dm Z    = 0   ;

  STORAGE_CLASS TYPE pm *p_a = &A[0] ;
  STORAGE_CLASS TYPE dm *p_b = &B[0] ;
  STORAGE_CLASS TYPE dm *p_z = &Z ;
  
  STORAGE_CLASS TYPE f ; 

  pin_down(&Z) ; 
  
  START_PROFILING; 
  
  for (f=0;f<2;f++)
    *p_z  += *p_a++ * *p_b++ ;
  
  END_PROFILING; 
  
  pin_down(&Z) ; 
  
  return(0)  ; 
}
