{ 

  Radix-2 DIT FFT Subroutine                                   R2DITFFT.DSP

     Performs Radix-2 DIT FFT

     Calling Parameters
         inplacereal = Real input data in scrambled order
         inplaceimag = All zeroes (real input assumed)
         twid_real = Twiddle factor cosine values
         twid_imag = Twiddle factor sine values
         groups = N/2
         bflys_per_group = 1
         node_space = 1

     Return Values
         inplacereal = Real FFT results in sequential order
         inplaceimag = Imaginary FFT results in sequential order

     Altered Registers
         I0,I1,I2,I3,I4,I5,L0,L1,L2,L3,L4,L5
         M0,M1,M2,M3,M4,M5
         AX0,AX1,AY0,AY1,AR,AF
         MX0,MX1,MY0,MY1,MR,SB,SE,SR,SI

     Altered Memory
         inplacereal, inplaceimag, groups, node_space,
         bflys_per_group, blk_exponent
}
.MODULE      radix2_dit_fft;

.CONST       logN=10, N=1024;          {Set constants for N-point FFT}
.EXTERNAL    twid_real, twid_imag;
.EXTERNAL    inplacereal, inplaceimag;
.EXTERNAL    groups, bflys_per_group, node_space;
.EXTERNAL    bfp_adj;
.ENTRY       fft_strt;

fft_strt:    
         CNTR=logN;                {Initialize stage counter}
         M0=0;
         M1=1;
         L1=0;
         L2=0;
         L3=0;
         L4=%twid_real;
         L5=%twid_imag;
         DO stage_loop UNTIL CE;       {Compute all stages in FFT}
             I0=^inplacereal;          {I0 x0 in 1st grp of stage}
             I2=^inplaceimag;          {I2 y0 in 1st grp of stage}
             SB=-2 ;                    {SB to detect data > 14 bits}
             SI=DM(groups);
             CNTR=SI;                  {CNTR = group counter}
             M4=SI;                    {M4=twiddle factor modifier}
             M2=DM(node_space);        {M2=node space modifier}
             I1=I0;
             MODIFY(I1,M2);            {I1 x1 of 1st grp in stage}
             I3=I2;
             MODIFY(I3,M2);            {I3 y1 of 1st grp in stage}
             DO group_loop UNTIL CE;
                 I4=^twid_real;        {I4  C of W0}
                 I5=^twid_imag;        {I5  (-S) of W0}
                 CNTR=DM(bflys_per_group);  {CNTR = butterfly counter}
                 MY0=PM(I4,M4),MX0=DM(I1,M0); {MY0=C,MX0=x1 }
                 MY1=PM(I5,M4),MX1=DM(I3,M0); {MY1=-S,MX1=y1}
                 DO bfly_loop UNTIL CE;
                     MR=MX0*MY1(SS),AX0=DM(I0,M0); {MR=x1(-S),AX0=x0}
                     MR=MR+MX1*MY0(RND),AX1=DM(I2,M0);
                                       {MR=(y1(C)+x1(-S)),AX1=y0}
                     AY1=MR1,MR=MX0*MY0(SS); {AY1=y1(C)+x1(-S),MR=x1(C)}
                     MR=MR-MX1*MY1(RND);     {MR=x1(C)-y1(-S)}
                     AY0=MR1,AR=AX1-AY1;     {AY0=x1(C)-y1(-S),}
                                             {AR=y0-[y1(C)+x1(-S)]}
                     SB=EXPADJ AR,DM(I3,M1)=AR; {Check for bit growth,}
                                                {y1'=y0-[y1(C)+x1(-S)]}
                     AR=AX0-AY0,MX1=DM(I3,M0),MY1=PM(I5,M4);
                                                {AR=x0-[x1(C)-y1(-S)],}
                                                {MX1=next y1,MY1=next (-S)}
                     SB=EXPADJ AR,DM(I1,M1)=AR; {Check for bit growth,}
                                                {x1'=x0-[x1(C)-y1(-S)]}
                     AR=AX0+AY0,MX0=DM(I1,M0),MY0=PM(I4,M4);
                                                {AR=x0+[x1(C)-y1(-S)],}
                                                {MX0=next x1,MY0=next C}
                     SB=EXPADJ AR,DM(I0,M1)=AR; {Check for bit growth,}
                                                {x0'=x0+[x1(C)-y1(-S)]}
                     AR=AX1+AY1;                {AR=y0+[y1(C)+x1(-S)]}
bfly_loop:           SB=EXPADJ AR,DM(I2,M1)=AR; {Check for bit growth,}
                                                {y0'=y0+[y1(C)+x1(-S)]}
                 MODIFY(I0,M2);        {I0 1st x0 in next group}
                 MODIFY(I1,M2);        {I1 1st x1 in next group}
                 MODIFY(I2,M2);        {I2 1st y0 in next group}
group_loop:      MODIFY(I3,M2);        {I3 1st y1 in next group}
             CALL bfp_adj;             {Compensate for bit growth}
             SI=DM(bflys_per_group);
             SR=ASHIFT SI BY 1(LO);
             DM(node_space)=SR0;       {node_space=node_space x 2}
             DM(bflys_per_group)=SR0;  {bflys_per_group= }
                                       {bflys_per_group x 2}
             SI=DM(groups);
             SR=ASHIFT SI BY -1(LO);
stage_loop:  DM(groups)=SR0;           {groups=groups  2}
         RTS;
.ENDMOD;

