******************************************************************************
* FILE: c5cxrad2.asm   --> macro file for radix 2 fft's based on 320c5x      *
*                                                                            *
* COPYRIGHT TEXAS INSTRUMENTS INC. 1990/1994                                 *
*                                                                            *
* WRITTEN BY: MANFRED CHRIST	   LAST UPDATE: 21. Mar 1994		     *
******************************************************************************
*                                                                            *
* MACRO 'COMBO5X' FOR THE COMPLEX, RADIX-2 DIT FFT                           *
*                                                                            *
* WRITTEN BY: MANFRED CHRIST	   LAST UPDATE: 21. Mar. 94		     *
* REVISION  : 2.00							     *
* CYCLES    : 33      WORDS: 33                                              *
* ORGANIATION OF THE INPUT DATA MEMORY: R1,I1,R2,I2,R3,I3,R4,I4              *
*                                                                            *
******************************************************************************
*                                                                            *
* The macros aren't DP independent. They require DP=0                        *
*                                                                            *
******************************************************************************
*                                                                            *
* THE MACRO 'COMBO2x' PERFORMS FOLLOWING CALCULATIONS:                       *
*                                                                            *
* R1  := [(R1+R2)+(R3+R4)]/4       INPUT               OUTPUT                *
* R2  := [(R1-R2)+(I3-I4)]/4       ------------------  ------------------    *
* R3  := [(R1+R2)-(R3+R4)]/4       AR0 =  7                                  *
* R4  := [(R1-R2)-(I3-I4)]/4       AR1 -> R1,I1        AR1 - > R5,I5         *
* I1  := [(I1+I2)+(I3+I4)]/4       AR2 -> R2,I2        AR2 - > R6,I6         *
* I2  := [(I1-I2)-(R3-R4)]/4       ARP-> AR3 -> R3,I3  ARP - > AR3 - > R7,I7 *
* I3  := [(I1+I2)-(I3+I4)]/4       AR4 -> R4,I4        AR4 - > R8,I8         *
* I4  := [(I1-I2)+(R3-R4)]/4                                                 *
*                                                                            *
* For a 16-point Radix 2 complex FFT the Macro 'COMBO5x' has to be           *
* repeated N/4 times (e.g. 4  times for a  16 point FFT).                    *
*                                                                            *
******************************************************************************
;^L
combo5x .macro		    ; REPEAT MACRO 'combo5x': N/4 times
	SPLK	#7,INDX     ; index register = 7
	LAR	AR1,DATAADD ; pointer to DATA	    r1,i1
	LAR	AR2,DATA2   ; pointer to DATA + 2   r2,i2
	LAR	AR3,DATA4   ; pointer to DATA + 4   r3,i3
	LAR	AR4,DATA6   ; pointer to DATA + 6   r4,i4
	splk	#N/4-1,BRCR ; execute N/4-1 times 'COMBO5x'
;
	RPTB	cmbend	  ;			       ARP AR1 AR2 AR3 AR4 AR5
;                                                       --- --- --- --- --- ---
        LACC    *,14,AR4  ; ACC := (R3)/4               4  R1  R2  R3  R4  T1
        SUB     *,14,AR5  ; ACC := (R3+R4)/4            5  R1  R2  R3  R4  T1
        SACH    *+,1,AR4  ; T1   = (R3-R4)/2            4  R1  R2  I3  R4  T2
;
        ADD     *+,15,AR5 ; ACC := (R3+R4)/4            5  R1  R2  R3  I4  T2
        SACH    *,1,AR2   ; T2   = (R3+R4)/2            2  R1  R2  R3  I4  T2
;
        ADD     *,14,AR1  ; ACC := (R2+R3+R4)/4         1  R1  R2  R3  I4  T2
        ADD     *,14      ; ACC := (R1+R2+R3+R4)/4      1  R1  R2  R3  I4  T2
        SACH    *+,0,AR5  ; R1  := (R1+R2+R3+R4)/4      5  I1  R2  R3  I4  T2
        SUB     *,16,AR3  ; ACC := (R1+R2-(R3+R4))/4    3  I1  R2  R3  I4  T2
        SACH    *+,0,AR5  ; R3  := (R1+R2-(R3+R4))/4    5  I1  R2  I3  I4  T2
;
        ADD     *,15,AR2  ; ACC := (R1+R2)/4            2  I1  R2  I3  I4  T2
        SUB     *,15,AR3  ; ACC := (R1-R2)/4            3  I1  R2  I3  I4  T2
        ADD     *,14,AR4  ; ACC := ((R1-R2)+(I3))/4     4  I1  R2  I3  I4  T2
        SUB     *,14,AR2  ; ACC := ((R1-R2)+(I3-I4))/4  2  I1  R2  I3  I4  T2
        SACH    *+,0,AR4  ; R2  := ((R1-R2)+(I3-I4))/4  4  I1  I2  I3  I4  T2
        ADD     *-,15,AR3 ; ACC := ((R1-R2)+ I3+I4 )/4  3  I1  I2  I3  R4  T2
        SUB     *,15,AR4  ; ACC := ((R1-R2)-(I3-I4))/4  4  I1  I2  I3  R4  T2
        SACH    *+,0,AR1  ; R4  := ((R1-R2)-(I3-I4))/4  1  I1  I2  I3  I4  T2
;
        LACC    *,14,AR2  ; ACC := (I1)/4               2  I1  I2  I3  I4  T2
        SUB     *,14,AR5  ; ACC := (I1-I2)/4            5  I1  I2  I3  I4  T2
        SACH    *,1,AR2   ; T2  := (I1-I2)/2            2  I1  I2  I3  I4  T2
        ADD     *,15,AR3  ; ACC := ((I1+I2))/4          4  I1  I2  I3  I4  T2
        ADD     *,14,AR4  ; ACC := ((I1+I2)+(I3))/4     4  I1  I2  I3  I4  T2
        ADD     *,14,AR1  ; ACC := ((I1+I2)+(I3+I4))/4  1  I1  I2  I3  I4  T2
        SACH    *0+,0,AR3 ; I1  := ((I1+I2)+(I3+I4))/4  3  R5  I2  I3  I4  T2
        SUB     *,15,AR4  ; ACC := ((I1+I2)-(I3+I4))/4  4  R5  I2  I3  I4  T2
        SUB     *,15,AR3  ; ACC := ((I1+I2)-(I3+I4))/4  3  R5  I2  I3  I4  T2
        SACH    *0+,0,AR5 ; I3  := ((I1+I2)-(I3+I4))/4  5  R5  I2  R7  I4  T2
;
        LACC    *-,15     ; ACC := (I1-I2)/4            5  R5  I2  R7  I4  T1
        SUB     *,15,AR2  ; ACC := ((I1-I2)-(R3-R4))/4  2  R5  I2  R7  I4  T1
        SACH    *0+,0,AR5 ; I2  := ((I1-I2)-(R3-R4))/4  5  R5  R6  R7  I4  T1
        ADD     *,16,AR4  ; ACC := ((I1-I2)+(R3-R4))/4  4  R5  R6  R7  I4  T1
cmbend: SACH	*0+,0,AR3 ; I4	:= ((I1-I2)+(R3-R4))/4	3  R5  R6  R7  R8  T1
        MAR     *,AR2     ; ARP=AR2
	.endm
;
******************************************************************************
*                                                                            *
*     MACRO 'ZEROI'    number of words/number of cycles: 10                  *
*                                                                            *
*        ARP=2 FOR INPUT AND OUTPUT                                          *
*        AR2 -> QR,QI,QR+1,...                                               *
*        AR3 -> PR,PI,PR+1,...                                               *
*                                                                            *
*        CALCULATE Re[P+Q] AND Re[P-Q]                                       *
*        QR'=(PR-QR)/2                                                       *
*        PR'=(PR+QR)/2                                                       *
*        PI'=(PI+QI)/2                                                       *
*        PI'=(PI-QI)/2                                                       *
*                                                                            *
*  version 2.00    from Manfred Christ	 update: 21. Mar 94		     *
*                                                                            *
******************************************************************************
ZEROI    .macro            ;                            AR1  AR2  ARP
         LACC    *,15,AR1  ; ACC := (1/2)(QR)           PR   QR    1
         ADD     *,15      ; ACC := (1/2)(PR+QR)        PR   QR    1
         SACH    *+,0,AR2  ; PR  := (1/2)(PR+QR)        PI   QR    2
         SUB     *,16      ; ACC := (1/2)(PR+QR)-(QR)   PI   QR    2
         SACH    *+        ; QR  := (1/2)(PR-QR)        PI   QI    2
;
         LACC    *,15,AR1  ; ACC := (1/2)(QI)           PI   QI    1
         ADD     *,15      ; ACC := (1/2)(PI+QI)        PI   QI    1
         SACH    *+,0,AR2  ; PI  := (1/2)(PI+QI)        PR+1 QI    2
         SUB     *,16      ; ACC := (1/2)(PI+QI)-(QI)   PR+1 QI    2
         SACH    *+        ; QI  := (1/2)(PI-QI)        PR+1 QR+1  2
         .endm
******************************************************************************
*                                                                            *
*     MACRO 'PBY2I'    number of words/number of cycles: 12                  *
*                                                                            *
*        PR'=(PR+QI)/2      PI'=(PI-QR)/2                                    *
*        QR'=(PR-QI)/2      QI'=(PI+QR)/2                                    *
*                                                                            *
*  version 2.00    from Manfred Christ	 update: 21. Mar. 94		     *
*                                                                            *
******************************************************************************
PBY2I    .macro            ;                            AR1  AR2  ARP
         LACC    *+,15,AR5 ;                            PR   QI    5
         SACH    *,1,AR2   ; TMP=QR                     PR   QI    2
;
         LACC    *,15,AR1  ; ACC := QI/2                PR   QI    1
         ADD     *,15      ; ACC := (PR+QI)/2           PR   QI    1
         SACH    *+,0,AR2  ; PR  := (PR+QI)/2           PI   QI    2
         SUB     *-,16     ; ACC := (PR-QI)/2           PI   QR    2
         SACH    *+,0,AR1  ; QR  := (PR-QI)/2           PI   QI    1
;
         LACC    *,15,AR5  ; ACC := (PI)/2              PI   QI    5
         SUB     *,15,AR1  ; ACC := (PI-QR)/2           PI   QI    1
         SACH    *+,0,AR5  ; PI  := (PI-QR)/2           PR+1 QI    5
         ADD     *,16,AR2  ; ACC := (PI+QR)/2           PR+1 QI    2
         SACH    *+        ; QI  := (PI+QR)/2           PR+1 QI+1  2
         .endm
*
******************************************************************************
*                                                                            *
*     MACRO 'PBY4J'    number of words/number of cycles: 16                  *
*                                                                            *
*  T=SIN(45)=COS(45)=W45                                                     *
*                                                                            *
*                                                                            *
*        PR'= PR + (W*QI + W*QR) = PR + W * QI + W * QR    (<- AR1)          *
*        QR'= PR - (W*QI + W*QR) = PR - W * QI - W * QR    (<- AR2)          *
*        PI'= PI + (W*QI - W*QR) = PI + W * QI - W * QR    (<- AR1+1)        *
*        QI'= PI - (W*QI - W*QR) = PI - W * QI + W * QR    (<- AR1+2)        *
*                                                                            *
*                                                                            *
*  version 2.00    from Manfred Christ	 update: 21. Mar. 94		     *
*                                                                            *
******************************************************************************
;
PBY4J    .macro            ; TREG= W                  AR5   PREG   AR1  AR2 ARP
         MPY      *+,AR5   ; PREG= W*QR/2               -   W*QR/2 PR   QI   5
         SPH      *,AR1    ; TMP = W*QR/2            W*QR/2 W*QR/2 PR   QI   1
         LACC     *,15,AR2 ; ACC = PR/2              W*QR/2 W*QR/2 PR   QI   2
         MPYS     *-       ; ACC = (PR-W*QR)/2       W*QR/2 W*QI/2 PR   QR   2
         SPAC              ; ACC = (PR-W*QI-W*QR)/2  W*QR/2 W*QI/2 PR   QR   2
         SACH     *+,0,AR1 ; QR  = (PR-W*QI-W*QR)/2  W*QR/2 W*QI/2 PR   QI   1
         SUB      *,16     ; ACC = (-PR-W*QI-W*QR)/2 W*QR/2 W*QI/2 PR   QI   1
         NEG               ; ACC =  (PR+W*QI+W*QR)/2 W*QR/2 W*QI/2 PR   QI   1
         SACH     *+       ; QR  =  (PR+W*QI+W*QR)/2 W*QR/2 W*QI/2 PI   QI   1
;
         LACC     *,15,AR5 ; ACC = (PI)/2            W*QR/2 W*QI/2 PI   QI   5
         SPAC              ; ACC = (PI-W*QI)/2       W*QR/2   -    PI   QI   5
         ADD      *,16,AR2 ; ACC = (PI-W*QI+W*QR)/2     -     -    PI   QI   2
         SACH     *+,0,AR1 ; QI  = (PI-W*QI+W*QR)/2     -     -    PI   QR1  1
         SUB      *,16     ; ACCU= (-PI-W*QI+W*QR)/2    -     -    PI   QR1  1
         NEG               ; ACCU=  (PI+W*QI-W*QR)/2    -     -    PI   QR1  1
         SACH     *+,0,AR2 ; PI  =  (PI+W*QI-W*QR)/2    -     -    PR1  QR1  2
         .endm
*
******************************************************************************
*                                                                            *
*     MACRO 'P3BY4J'   number of words/number of cycles: 16                  *
*                                                                            *
*  version 2.00    from: Manfred Christ   update: 21. Mar. 94		     *
*                                                                            *
*   ENTRANCE IN THE MACRO: ARP=AR2                                           *
*                          AR1->PR,PI                                        *
*                          AR2->QR,QI                                        *
*                          TREG=W=COS(45)=SIN(45)                            *
*                                                                            *
*        PR'= PR + (W*QI - W*QR) = PR + W * QI - W * QR    (<- AR1)          *
*        QR'= PR - (W*QI - W*QR) = PR - W * QI + W * QR    (<- AR2)          *
*        PI'= PI - (W*QI + W*QR) = PI - W * QI - W * QR    (<- AR1+1)        *
*        QI'= PI + (W*QI + W*QR) = PI + W * QI + W * QR    (<- AR1+2)        *
*                                                                            *
*       EXIT OF THE MACRO: ARP=AR2                                           *
*                          AR1->PR+1,PI+1                                    *
*                          AR2->QR+1,QI+1                                    *
*                                                                            *
******************************************************************************
P3BY4J   .macro            ; TREG= W                  AR5   PREG   AR1  AR2 ARP
         MPY    *+,AR5     ; PREG= W*QR/2               -   W*QR/2 PR   QI   5
         SPH    *,AR1      ; TMP = W*QR/2            W*QR/2 W*QR/2 PR   QI   1
         LACC   *,15,AR2   ; ACC = PR/2              W*QR/2 W*QR/2 PR   QI   2
         MPYA   *-         ; ACC = (PR+W*QR)/2       W*QR/2 W*QI/2 PR   QR   2
         SPAC              ; ACC = (PR-W*QI+W*QR)/2  W*QR/2 W*QI/2 PR   QR   2
         SACH   *+,0,AR1   ; QR' = (PR-W*QI+W*QR)/2  W*QR/2 W*QI/2 PR   QI   1
         SUB    *,16       ; ACC = (-PR-W*QI+W*QR)/2 W*QR/2 W*QI/2 PR   QI   1
         NEG               ; ACC =  (PR+W*QI-W*QR)/2 W*QR/2 W*QI/2 PR   QI   1
         SACH   *+         ; PR' =  (PR+W*QI-W*QR)/2 W*QR/2 W*QI/2 PI   QI   1
;
         LACC   *,15,AR5   ; ACC = (PI)/2            W*QR/2 W*QI/2 PI   QI   5
         APAC              ; ACC = (PI+W*QI)/2       W*QR/2   -    PI   QI   5
         ADD    *,16,AR2   ; ACC = (PI+W*QI+W*QR)/2     -     -    PI   QI   2
         SACH   *0+,0,AR1  ; QI' = (PI+W*QI+W*QR)/2     -     -    PI   QR5  1
         SUB    *,16       ; ACCU= (-PI+W*QI+W*QR)/2    -     -    PI   QR5  1
         NEG               ; ACCU=  (PI-W*QI-W*QR)/2    -     -    PI   QR5  1
         SACH   *0+,0,AR2  ; PI' =  (PI-W*QI-W*QR)/2    -     -    PR5  QR5  2
         .endm
;
******************************************************************************
*                                                                            *
*     MACRO 'stage3'   number of words/number of cycles: 54                  *
*                                                                            *
*  version 2.00    from: Manfred Christ   update: 21. Mar. 94		     *
*                                                                            *
******************************************************************************
stage3   .macro num
	 SPLK	#:num:-1,BRCR  ; execute 'num'-1 times 'stage3'
	 SPLK	#9,INDX        ; index register = 9
	 LAR	AR1,DATAADD    ; ar1 -> DATA
	 LAR	AR2,DATA8      ; ar2 -> DATA+8
         LT     cos45
         RPTB   stage3e
         ZEROI
         PBY4J
         PBY2I
         P3BY4j
	 .if	N=8
	 retd		       ; delayed return
	 mar	 *,ar6	       ; ARP=ar6
	 nop
	 .endif
stage3e: .set   $-1
         .endm
;
******************************************************************************
*                                                                            *
*  MACRO:  'BUTTFLYI'      generell butterfly radix 2 for 320C5x             *
*                                                                            *
*  version 2.00    from Manfred Christ	 update: 21. Mar. 94		     *
*                                                                            *
*  THE MACRO 'BUTTFLYI' REQUIRES 18 WORDS AND 18 INSTRUCTIONS                *
*                                                                            *
*  Definition: ARP -> AR2  (input)  ARP -> AR2      (output)                 *
*                                                                            *
*  Definition: AR1 -> QR   (input)  AR1 -> QR+1     (output)                 *
*  Definition: AR2 -> PR   (input)  AR2 -> PR+1     (output)                 *
*  Definition: AR3 -> Cxxx (input)  AR3 -> Cxxx+1   (output)  --> WR=cosine  *
*  Definition: AR4 -> Sxxx (input)  AR4 -> Sxxx+1   (output)  --> WI=sine    *
*  Definition: AR5 -> temporary variable (unchanged)                         *
*                                                                            *
*  uses index register                                                       *
*                                                                            *
*        PR' = (PR+(QR*WR+QI*WI))/2       WR=COS(W)   WI=SIN(W)              *
*        PI' = (PI+(QI*WR-QR*WI))/2                                          *
*        QR' = (PR-(QR*WR+QI*WI))/2                                          *
*        QI' = (PI-(QI*WR-QR*WI))/2                                          *
*                                                                            *
******************************************************************************
BUTTFLYI .macro
;                                            (contents of register after exec.)
;                                            TREG AR1  AR2  AR3 AR4 ARP
  RPTB  btflyend  ;                          ---- ---  ---  --- --- ---
  LT    *+,AR3    ;TREG:= QR                   QR PR   QI    C   S   3
  MPY   *,AR2     ;PREG:= QR*WR/2              QR PR   QI    C   S   2
  LTP   *-,AR4    ;ACC := QR*WR/2              QI PR   QR    C   S   4
  MPY   *,AR3     ;PREG:= QI*WI/2              QI PR   QR    C   S   3
  MPYA  *+,AR2    ;ACC := (QR*WR+QI*WI)/2      QR PR   QR    C+1 S   2
;                  PREG:= QI*WR
  LT    *,AR5     ;TREG = QR                   QR PR   QR    C+1 S   5
  SACH  *,1,AR1   ;H0  := (QR*WR+QI*WI)        QR PR   QR    C+1 S   1
;
  ADD   *,15      ;ACC := (PR+(QR*WR+QI*WI))/2 QR PR   QR    C+1 S   1
  SACH  *+,0,AR5  ;PR  := (PR+(QR*WR+QI*WI))/2 QR PI   QR    C+1 S   5
  SUB   *,16,AR2  ;ACC := (PR-(QR*WR+QI*WI))/2 QR PI   QR    C+1 S   2
  SACH  *+,0,AR1  ;QR  := (PR-(QR*WR+QI*WI))/2 QR PI   QI    C+1 S   1
;
  LAC   *,15,AR4  ;ACC := PI     /PREG=QI*WR   QI PI   QI    C+1 S   4
  MPYS  *+,AR2    ;PREG:= QR*WI/2              QI PI   QI    C+1 S+1 2
;                 ;ACC := (PI-QI*WR)/2
  APAC            ;ACC := (PI-(QI*WR-QR*WI))/2 QI PI   QI    C+1 S+1 2
  SACH  *+,0,AR1  ;QI  := (PI-(QI*WR-QR*WI))/2 QI PI   QR+1  C+1 S+1 1
  NEG             ;ACC :=(-PI+(QI*WR-QR*WI))/2 QI PI   QR+1  C+1 S+1 1
  ADD   *,16      ;ACC := (PI+(QI*WR-QR*WI))/2 QI PI   QR+1  C+1 S+1 1
btflyend:
  SACH  *+,0,AR2  ;PI  := (PI+(QI*WR-QR*WI))/2 QI PR+1 QR+1  C+1 S+1 2
  .endm
;
******************************************************************************
*                                                                            *
*  MACRO:  'BUTT_C5x'      optimized butterfly radix 2 for 320C5x            *
*                                                                            *
*  version 2.00    from Manfred Christ	 update: 21. Mar  94		     *
*                                                                            *
* The kernel of the macro 'butt_c5x' is executed in 17 cycles!               *
*                                                                            *
*  Definition: ARP -> AR2  (input)  ARP -> AR2      (output)                 *
*                                                                            *
*  Definition: AR1 -> QR   (input)  AR1 -> QR+1     (output)                 *
*  Definition: AR2 -> PR   (input)  AR2 -> PR+1     (output)                 *
*  Definition: AR3 -> Cxxx (input)  AR3 -> Cxxx+1   (output)  --> WR=cosine  *
*  Definition: AR4 -> Sxxx (input)  AR4 -> Sxxx+1   (output)  --> WI=sine    *
*  Definition: AR5 -> temporary variable (unchanged)                         *
*                                                                            *
*  uses index register: INDX=2                                               *
*                                                                            *
*        PR' = (PR+(QR*WR+QI*WI))/2       WR=COS(W)   WI=SIN(W)              *
*        PI' = (PI+(QI*WR-QR*WI))/2                                          *
*        QR' = (PR-(QR*WR+QI*WI))/2                                          *
*        QI' = (PI-(QI*WR-QR*WI))/2                                          *
*                                                                            *
* The instruction 'lt *+,ar3' has been placed outside of the RPTB-block
* to reduce from 17 to 2 cycles! Therefore the Index register INDX has to
* be used (with value 2)!
******************************************************************************
;                                       (contents of register after exec.)
;                                              TREG  AR1  AR2  AR3 AR4 ARP
;                                              ----  ---  ---  --- --- ---
butt_c5x .macro   ;                             QR   PR   QR    C   S   2
  lt    *+,ar3    ;TREG:= QR                    QR   PR   QI    C   S   3
;
  rptb  btflyend
  mpy   *,ar2     ;PREG:= QR*WR/2               QR   PR   QI    C   S   2
  ltp   *-,ar4    ;ACC := QR*WR/2               QI   PR   QR    C   S   4
  mpy   *,ar3     ;PREG:= QI*WI/2               QI   PR   QR    C   S   3
  mpya  *+,ar2    ;ACC := (QR*WR+QI*WI)/2       QR   PR   QR    C+1 S   2
;                  PREG:= QI*WR
  lt    *,ar5     ;TREG = QR                    QR   PR   QR    C+1 S   5
  sach  *,1,ar1   ;H0  := (QR*WR+QI*WI)         QR   PR   QR    C+1 S   1
;
  add   *,15      ;ACC := (PR+(QR*WR+QI*WI))/2  QR   PR   QR    C+1 S   1
  sach  *+,0,ar5  ;PR  := (PR+(QR*WR+QI*WI))/2  QR   PI   QR    C+1 S   5
  sub   *,16,ar2  ;ACC := (PR-(QR*WR+QI*WI))/2  QR   PI   QR    C+1 S   2
  sach  *0+,0,ar1 ;QR  := (PR-(QR*WR+QI*WI))/2  QR   PI   QR+1  C+1 S   1
;
  lac   *,15,ar4   ;ACC := PI     /PREG=QI*WR   QI   PI   QR+1  C+1 S   4
  mpys  *+,ar2     ;PREG:= QR*WI/2              QI   PI   QR+1  C+1 S+1 2
;                  ;ACC := (PI-QI*WR)/2
  lta   *-         ;ACC := (PI-(QI*WR-QR*WI))/2 QR1  PI   QI    C+1 S+1 2
  sach  *0+,0,ar1  ;QI  := (PI-(QI*WR-QR*WI))/2 QR1  PI   QI+1  C+1 S+1 1
  neg              ;ACC :=(-PI+(QI*WR-QR*WI))/2 QR1  PI   QI+1  C+1 S+1 1
  add   *,16       ;ACC := (PI+(QI*WR-QR*WI))/2 QR1  PI   QI+1  C+1 S+1 1
btflyend:
  sach  *+,0,ar3   ;PI  := (PI+(QI*WR-QR*WI))/2 QI   PR+1 QR+1  C+1 S+1 2
  .endm
;
do_btfly .macro  num
	 calld	 macros 	; delayed call follows
	 lacl	 #:num:-2	; make it DP-independend
	 samm	 BRCR		; execute ZEROI + num-2 times BUTTFLYI
; delayed call is executed here
         .endm

do_loops .macro  num
         .if     num>=0102h
         lacc    #:num:-2
         .else
	 lacl	 #:num:-2	 ; make it DP-independend
         .endif    
	 samm	 BRCR		 ; execute ZEROI + num-2 times BUTTFLYI
         .endm

ex_btfly .macro
macros:  ZEROI                   ; execute macro ZEROI
         lacl #2                 ; make it DP-independend
         samm INDX               ; pmst(indx)=0 INDX=2
         butt_c5x                ; macro requires index=2
         lamm  ar0               ; pmst(indx)=1  -> ar0 is index
         samm  INDX              ; make macro DP-independend!!! 
         mar   *,ar2             ; arp=index
         mar   *-                ; correct pointer ar2
; delayed 'RET' follows !!
         retd                    ; delayed return
         mar     *0+,ar1         ; AR2 += indx
         mar     *0+,ar6         ; AR1 += indx
; delayed 'RET' is executed here !!
         .endm

bitrev	 .macro
	 LAR	 AR3,DATAADD	 ; TRANSFER  64 WORDS FROM 'input' to 'data'
	 splk	 #N,INDX	 ; index register = N
	 RPT	 NN2		 ; N*2 TIMES
         BLDD    #INPUT,*BR0+
	 .endm

stdmacro .macro  stage,num,loops,add
         LAR	 ar0,#num*2	 ; index register = num * 2
         LAR     AR1,DATAADD     ; ar1 -> DATA
	 LAR	 AR2,DATA:add:   ; ar2 -> DATA+16
	 LAR	 ar6,#:loops:	 ; ar6 is loopcounter -> repeat loop+1 times
loop?:	 LAR	 AR3,cos:stage:  ; start of cosine in this stage
	 LAR	 AR4,sin:stage:  ; start of sine in this stage
	 do_btfly num		 ; execute ZEROI & 7 times BUTTFLYI
	 BANZ	 loop?,*-,ar2	 ; repeat loop until ar6==0
	 .endm

laststag .macro  stg,offset
	 LAR	 AR1,DATAADD	      ; ar1 -> DATA
      .if     N>=1024		      ; for N=1024
	 LAR	 AR2,#DATA+N	      ; ar2 -> DATA+1024
	 LAR	 AR3,#SINES:stg:+N/4  ; start of cosine in stage 10
	 LAR	 AR4,#SINES:stg:      ; start of sine in stage	 10
      .else			      ; for N<=512
	 LAR	 AR2,DATA:offset:     ; ar2 -> DATA+offset
	 LAR	 AR3,cos:stg:	      ; start of cosine in stage 'stg'
	 LAR	 AR4,sin:stg:	      ; start of sine in stage	 'stg'
      .endif
; the next two macros finish the FFT
	 do_loops N/2		      ; setup for N/2 butterflies
	 ex_btfly		      ; execute the butterflies
MACROLEN .set    $-macros
         .endm
; end of file: c5cxrad2.asm
