*  benchmark program  : c5cx1024.asm
* 
*  benchmark suite    : DSP-kernel
*
*  description        : benchmarking of an integer FFT
*                       Radix 2, In-place, Decimation-in-time Complex FFT Macro
*
*			The code c5cx1024.asm presents a 1024 point 
*			Decimation in Time, radix-2, Complex FFT
*
*    PROGRAMSEQUENCE: 0. INITIALISATION FOR FFT/COEFF      
*                     1. INPUT NEW DATA INTO 'INPUT'      
*                     2. CALL SUBROUTINE FFT               
*                     2.1. BITREVERSAL FROM INPUT TO DATA  
*                     2.2. FFT WITH WORK SPACE DATA        
*                     3. OUTPUT THE RESULTS FROM DATA      
*
*    USED  REGISTER: INDX,AR0..AR6,ACCU,PREG,TREG0,PMST,BRCR                  
*                    2 Stacklevel, Block B2                                   
*                                                                             
*    PROGRAM MEMORY:  236 WORDS ('FFTLEN') WITHOUT INITIALIZATION 
*                                                                             
*    COEFFICIENTS  :   16 BITS  (Q15 Format)  SCALING:   1/2^4                
*                                                                             
*    CYCLES	   : 80696 CYLCES =>  4034.8 us  2824.4	us  2017.4 us	       
*			  T-CYCLE =>	  50 ns      35 ns	25 ns	      
*                                                                             
*    MEMORY USAGE: FFT PROGRAM    [ 00600H -  006edH ]                        
*                  INPUT DATA     [ 02000H -  027ffH ]                        
*                  DATA WORKSPACE [ 00800H -  00fffH ]                        
*                  TWIDDLES       [ 01000H -  015ecH ]                        
*                                                                             
*    INPUT DATA AT ADDRESS 2000h-27ffh:                                       
*    ----------------------------------                                       
*    THE DATA ARE STORED IN 'input' IN THE SEQUENCE: X(0),X(1),...,X(1023)    
*                                                    Y(0),Y(1),...,Y(1023)    
*                                                                             
*    OUTPUT DATA AT ADDRESS 2000h-27FFh:                                      
*    -----------------------------------                                      
*    THE DATA ARE STORED IN SECTION 'fftdata' IN THE SEQUENCE:                
*    X(0),Y(0),X(1),Y(1),... ... ,X(1023),Y(1023)                             
*
*    REFERENCE :    WRITTEN BY: MANFRED CHRIST  REVISION: 2.00   
*
*		
*    THE PROGRAM IS BASED ON THE BOOK 'DIGITAL SIGNAL PROCESSING APPLICATIONS'
*    FROM TEXAS INSTRUMENTS P. 69. IT IS OPTIMIZED FOR THE TMS320C5x INCLUDING
*    BIT REVERSAL ADDRESSING MODE.           
*                                                                                
*
*  func. verification : 
*
*
*
*  organization       : Aachen University of Technology - IS2
*                     : DSP Tools Group
*                     : phone   : +49(241)807887
*                     : fax     : +49(241)8888195
*                     : e-mail  : zivojnov@ert.rwth-aachen.de
*
*  author             : Juan Martinez Velarde
*
*  history            : 24-03-94 - assembler reference code creation 
*                       24-03-94 - assembler reference code profiling

         .file      "c5cx1024.asm"
         .title     "1024 point DIT Radix-2, Complex FFT"
         .width     120
N        .set       1024       ; NUMBER OF POINTS FOR FFT

         .include init-fft.asm

         .sect    "fftprog"

; FFT CODE WITH BIT-REVERSED INPUT SAMPLES / ARP=AR3 / DP=0

START_PROFILING:

FFT:	 bitrev 	       ; macro to perform the bitreversal
STAGE1:  combo5x	       ; FFT CODE for STAGES 1 and 2
STAGE3:  stage3   128	       ; FFT CODE FOR STAGE  3 / ARP=AR2
STAGE4:  stdmacro 4,8,63,16    ; FFT CODE FOR STAGE  4 / ARP=AR2
STAGE5:  stdmacro 5,16,31,32   ; FFT CODE FOR STAGE  5 / ARP=AR2
STAGE6:  stdmacro 6,32,15,64   ; FFT CODE FOR STAGE  6 / ARP=AR2
STAGE7:  stdmacro 7,64,7,128   ; FFT CODE FOR STAGE  7 / ARP=AR2
STAGE8:  stdmacro 8,128,3,256  ; FFT CODE FOR STAGE  8 / ARP=AR2
STAGE9:  stdmacro 9,256,1,512  ; FFT CODE FOR STAGE  9 / ARP=AR2
STAGE10: laststag 10,1024      ; FFT CODE FOR STAGE 10 / ARP=AR2

END_PROFILING:

FFTLEN   .set    $-FFT
         .end
