{   Single Precision FIR Direct Form Filter Subroutine           FIRDIRFM.DSP
    Calling Parameters
         I0 -> Oldest Input Data Value, x(n-N+1), in Delay Line
         L0 = Filter Length (N) or Taps
         I4 -> Beginning of Filter Coefficient Table
         L4 = Filter Length (N) or Taps
         M0, M4 = 1
         CNTR = Filter Length - 1 (N - 1)
    Return Values
         MR1 = Filter Output y(n)
         I0 -> Oldest Input Data Value in Delay Line
         I4 -> Beginning of Filter Coefficient Table
    Altered Registers
         MX0, MY0, MR
         I0, L0, M0
         I4, L4, M4
    Reference 
         Digital Signal Processing Laboratory 
         Chapter 6, pp. 6.3.1. (modified)
}

.MODULE                      fir_arc;
.CONST                       N=16 ;          {filter length} ; 
.VAR/PM/RAM/ABS=0x0          coeff[N];
.INIT                        coeff: 0x100, 0x200, 0x300, 0x400, 0x500, 0x600,
                                    0x700, 0x800, 0x900, 0xa00, 0xb00, 0xc00,
                                    0xd00, 0xe00, 0xf00, 0x1000;
.VAR/DM/RAM                  data[N];
.VAR/DM/RAM		     input ; 
.INIT                        data:1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16;
.INIT			     input:100 ; 

.ENTRY fir_arc_;

fir_arc_:  
	 ENA M_MODE ; 
         I0=^data+15; L0=%data; M0=-1 ;
	 I1=^data+14; L1=0;     M1=0 ; 
         I2=^input;
         I4=^coeff+15; L4=%coeff; M4=-1;

START_PROFILING:

         CNTR = N-1 ;
         MR=0, MX0=DM(I0,M1), MY0=PM(I4,M4); 
	 DO conv UNTIL CE;
         MR=MR+MX0*MY0(SS), MX0=DM(I1,M0), MY0=PM(I4,M4);
conv:                       DM(I0,M0)=MX0;
         MR=MR+MX0*MY0(SS), MX0=DM(I2,M1);
                            DM(I0,M0)=MX0;

END_PROFILING:

end:     JUMP START_PROFILING ; 

.ENDMOD;
