;  benchmark program  : fir_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description        : benchmarking of a FIR
;
;  reference          : none
;
;  func. verification : with sim5x simulator 
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 18-03-94 - assembler reference code creation 
;                       18-03-94 - assembler reference code profiling

        .mmregs

        .file  "fir_arc.asm"
        .title "fir filter assembler reference code"
	.width 120

X        .set         0380h     ; Newest Data Sample, 
Y        .set         07Eh      ; filter output address

N        .set         16        ; number of taps

	.sect "coeff"
	.word 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16
;             ^
;           0x0fe00

COEFFP        .set        0fe00h    
    	                            
	.sect "data"
	.word 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
;             ^                             ^
;            0x380                        0x38f

LASTAP        .set        038Fh

        .sect ".text"
INIT:
        LAR        AR1,#X           ; AR1 -> newest data sample address
        MAR        *,AR1            ; ARP = AR1
        LACC        #100h           ; assume newest value is 0x100
        SACL        *,0             ; store X value

        MAR        * ,AR5           ; ARP = AR5
        LAR        AR5,#LASTAP      ; point to oldest sample
        
START_PROFILING:

        RPTZ        #N-1	    ; N tap filter
        MACD        COEFFP,*-       ; PREG = *AR5-- * coeff++

        APAC                        ; add P register to Accu
        LAR        AR1,#Y           ; load the filter output address
        MAR        *,AR1            ; ARP = AR1
        SACL        *,0,AR5         ; store the lower 16 bits of the
                                    ; accumulator as the filter output Y

END_PROFILING:

END:        B INIT

        .end
