;  benchmark program  : biq_one_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description        : benchmarking of an One Section IIR Filter
;
;  reference          : TMS320C5x User's Guide, pp. 7-40 and 7-41
;
;  func. verification : with sim5x simulator
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 23-03-94 - assembler reference code creation, profiling

        .mmregs

        .file  "biq_one_arc.asm"
        .title "iir one biquad filter assembler reference code"       
	.width 120

	.sect	".data"

STATES          .set    060h	; B2 
		.word	0	; w(n)
		.word 	0	; w(n-1)
		.word	0	; w(n-2)
STATES_END	.set    062h
COEFF		.set	063h
		.word 	01000h	; a2
		.word	01000h	; a1
		.word	01000h	; b2
		.word	01000h	; b1
		.word	05000h	; b0
X_ADD		.set    068h	
		.word 	0002h	; input sample
Y_ADD		.set	069h
		.word	0	; output sample

        .sect ".text"
INIT:	     			
				; setup pointers

	LAR	AR1,#STATES_END ; AR1 points to w(n-2)
	LAR	AR2,#COEFF	; AR2 points to first coeff
	LAR	AR3,#X_ADD	; AR3 points to input sample
	LAR	AR4,#Y_ADD	; AR4 points to output sample
	MAR	*,AR3		; ARP = AR3 
	LDP	#0		; dp = 0 
				; setup variables
	ZPR			; clear P register
	SPLK	#2,INDX		; Setup Index register        

START_PROFILING:

	LACC	*,15,AR1	; Get Q15 input
	LT	*-,AR2		; T = w(n-2)
	MPYA	*+,AR1		; a  = x(n), p = -w(n-2)*a2
	LTA	*-,AR2		; a += -w(n-2)*a2, T = w(n-1)	
	MPY	*+		; p = -w(n-1)*a1
	LTA	*+,AR1		; a += -w(n-1)*a1, T = b2
	SACH	*0+,1		; save w(n)
	MPY	*-		; p = w(n-2)*b2
	LACL	#0		; acc = 0 
	LTD	*-,AR2		; T = w(n-1), w(n-2) = w(n-1)
	MPY	*+,AR1		; a += w(n-2)*b2, p = w(n-1)*b1
	LTD	*-,AR2		; T = w(n), w(n-1) = w(n)
	MPY	*+,AR1		; a += w(n-1)*b1, p = w(n)*b0

	LTA	*,AR4		; final accumulation
	SACH	*,1		; save output

END_PROFILING:

END:        B END

        .end
