;  benchmark program  : lms_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description        : benchmarking of a LMS
;
;
;  reference : TMS320C50 Users Guide, p.7-39 Adaptive FIR 
;
;  func. verification : 
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 12-04-94 - assembler reference code creation 
;                       12-04-94 - assembler reference code profiling

	   .file "lms_arc.asm"
	   .title "LMS Adaptive Filter Assembler Reference Code" 

	   .width 150 
	   .mmregs

COEFFP .set    02000h	   ; Program Memory Address of the coeff. in S/A RAM
COEFFD .set    02000h	   ; Data memory address of the coeff. in S/A RAM

ONE    .set    07Ah	   ; Constant One.	       (DP=0).
BETA   .set    07Bh	   ; Adaptation constant.      (DP=0).
ERR    .set    07Ch	   ; Signal error.	       (DP=0).
ERRF   .set    07Dh	   ; Error function.	       (DP=0).
Y      .set    07Eh	   ; Filter output.	       (DP=0).
X      .set    037Fh	   ; Newest Data sample.
FRSTAP .set    0380h	   ; Next newest data sample.
LASTAP .set    038fh	   ; Oldest data sample.
N      .set    16	   ; tap number


	.sect	".values"
	.word 1,4096,0,0,0

	.sect	".coeff"
	.word 1,1,1,1,1,1,1,1
        .word 1,1,1,1,1,1,1,1

	.sect	".data"
	.word 2,0,0,0,0,0,0,0
        .word 0,0,0,0,0,0,0,0

	.sect ".text"

INIT

;
; Finite impulse response adaptive (FIR) filter.
;

; 
;  Setup pointers etc
;

ADPFIR	   
	   ZPR		   	; Clear P register.
           LACC    #1,14  	; Load output rounding bit.
	   LAR	   AR1,#ERR
	   LAR	   AR3,#LASTAP 	; Point to oldest sample.
	   LAR     AR6,#FRSTAP
	   LAR     AR4,#ERRF
	   LAR	   AR5,#BETA

 	   MAR	   *,AR3   	; ARP = AR3

START_PROFILING

;                  i=N-1
; FIR Filtering y = SUM {c(i)*x(n-i)}
;                   i=0

FIR	   RPTZ	   #(N-1)
	   MACD    COEFFP,*- ; FIR filtering
	   APAC	             ; add P register to accumulator 
	   SACH    Y,1      ; Store the filter output
	   NEG		    ; Acc = -y(n)
	   LAR	   AR3,#X   ; AR3 points to newest data sample
	   ADD	   *,15     ; Add the newest input sample.
	   SACH    ERR,1    ; err(n) = x(n) - y(n)
	   DMOV    *        ; Include newest sample
;
; LMS Adaption of Filter Coefficients.
;
	   LT	   ERR      ; T = err
	   MPY	   BETA     ; P = beta*err(i)
	   PAC		    ; errf(i) = beta * err(i) 
                            ; [load accu with product register] 
	   ADD	   ONE,14   ; Round the results.
	   SACH    ERRF,1   ; Save errf(i)
;
	   LACC    #(N-2)
	   SAMM    BRCR        ; N-1 coefficients to update
;				 in the loop.

	   LAR	   AR2,#COEFFD ; Point to the coefficients.
	   LAR	   AR3,#LASTAP ; Point to the data samples.
	   LT	   ERRF	       ; T = errf = beta * err(i)

	   MPY	   *-,AR2      ; P = 2*beta*err(i)*x(i-N+1)
;
	   RPTB    LOOP-1      ; For I=0,I<N-1,I++
ADAPT	   ZALR    *,AR3       ; Zero Low Accu 
                               ; Load High Accu (Round) with ak(i).
	   MPYA    *-,AR2      ; P = 2*beta*err(i)*x(i-k-1)
;			       ; Acc = ak(i) + 2*beta*err(i)*x(i-k)
	   SACH    *+	       ; Store ak(i+1)
;
LOOP	   ZALR    *,AR2       ; Finally update last coeff. a0(i)
	   APAC 	       ; Acc = a0(i) + 2*beta*err(i)*x(i)
	   SACH    *+	       ; Save a0(i+1)

END_PROFILING

END

	   B	INIT		; wait forever
