;  benchmark program  : mat1x3_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description:         1x3 matrix - multiply benchmarking
; 
;                |h11 h12 h13|   |x1|   |y1|   | h11*x1+h12*x2+h31*x3 |
;                |h21 h22 h23| x |x2| = |y2| = | h21*x1+h22*x2+h23*x3 |
;                |h31 h32 h33|   |x3|   |y3|   | h31*x1+h32*x2+h33*x3 |
;  
;  Elements are to store in following order:
;  
;  matrix h[9]={h11,h12,h13, h21,h22,h23, h31,h32,h33}
;  vector x[3]={x1,x2,x3}
;  vector y[3]={y1,y1,y3}
;
;  reference :          TI_BBS' c5xug/mtrxmpy.asm (modified)
;                       TMS320C5x User's Guide p.7-11.
;
;  func. verification : from sim5x simulator
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 06-04-94 - assembler reference code creation 
;                       07-04-94 - assembler reference code profiling

        .mmregs

        .file  "mat1x3_arc.asm"
        .title "[3x3][1x3] matrix multiply assembler reference code"
	.width 120

X_VEC	.set	0fe00h
H_MAT	.set	060h
Y_VEC	.set    070h

; following sections are defined in the cmd file !

; sect vector_x should be loaded in PAGE 0 (B0)
	.sect 	"vector_x"
	.word 	1,2,3

; sect matrix_h and vector_y should be loaded in PAGE 1 (B2)
	.sect	"matrix_h"
	.word	1,2,3,3,2,1,1,2,3

	.sect	"vector_y"
	.word	0,0,0

        .sect ".text"
INIT:				; program entry point
	LDP	#0
	LAR	AR1,#X_VEC	; POINTS TO X_VEC
	LAR	AR2,#H_MAT	; POINTS TO H_MAT
	LAR	AR3,#Y_VEC	; POINTS TO Y_VEC
	MAR	*,AR2		; ARP = AR2 
	SPLK	#3,INDX 	; ROW SIZE

START_PROFILING:

	SMMR	AR1,BMAR	;BMAR -> X(0)
	SPLK	#2,BRCR		;RPT BLOCK FOR THE 3 ROWS
	SAR	AR2,AR5 	;AR5 ->  H(0,0)

	RPTB	ELOOP		;ARP MUST BE AR2 IN THE LOOP
	SAR	AR5,AR2 	;AR2 -> H(K,0)
	RPTZ	#2		;RPT NXT INSTR 3 TIMES PRECEDED 
                                ;BY CLEARING OF ACC AND PREG
	MADS	*+		;ACC = X(J) * H(K,J)
	APAC			;FINAL ACCUMULATION

	MAR	*,AR5		;ARP=AR5
	MAR	*0+,AR3		;UPDATE AR5 BY INDX, ARP=AR3
ELOOP: 	SACL	*+,0,AR2	;SAVE Y(J), ARP=AR2

END_PROFILING:

END:        B END

        .end
