	section	mat1x3_arc2
	opt cc
	page 132,66,3,3


; **************************************************
; /*
;  * benchmark program:   mat1x3_arc2.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  * 
;  * description:         1x3 matrix - multiply benchmarking
;  *
;  *               |h11 h12 h13|   |x1|   |y1|   | h11*x1+h12*x2+h31*x3 |
;  *               |h21 h22 h23| x |x2| = |y2| = | h21*x1+h22*x2+h23*x3 |
;  *               |h31 h32 h33|   |x3|   |y3|   | h31*x1+h32*x2+h33*x3 |
;  * 
;  * Element are to store in following order:
;  * 
;  * matrix h[9]={h11,h12,h13, h21,h22,h23, h31,h32,h33}
;  * vector x[3]={x1,x2,x3}
;  * vector y[3]={y1,y1,y3}
;  * 
;  * reference code: Motorola's Dr.Bub/56100/source.1 program B.2.21 
;  *
;  * f. verification:      with printf function
;  * 
;  * organization:         Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             06-04-94 creation  (Martinez Velarde)
;  *                      06-04-94 profiling (Martinez Velarde)
;  */

	org	x:
F___h0
	dc .1	;h11
	dc .1	;h12
	dc .1	;h13
	dc .1	;h21
	dc .1	;h22
	dc .1	;h23
	dc .1	;h31
	dc .1	;h32
	dc .1	;h33
F___x1
	dc .2	;x1
	dc -.2	;x2
	dc .9	;x3
F___y2
	ds 3	; save 3 spaces for output vector y = {y1, y2, y3}

	org	p:

	global	Fmain
Fmain
	move	ssh,x:(r2)+			     ;save ssh
	move	m0,x:(r2)+			     ;save m0 register
        move    #F___x1,r0                           ;point to vec X
        move    #F___y2,r1                           ;point to vec Y
        move    #F___h0,r3                           ;point to mat H
        move    #2,m0                                ;addrb mod 3
	nop

ASM_APP_0
	START_PROFILING:
ASM_APP_1

        move              x:(r0)+,y0  x:(r3)+,x0    ;y0=h11;x0=x1
        mpy     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;h11*x1
        mac     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;+h12*x2
        macr    y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;+h13*x3
        move              a,x:(r1)+                 ;store y1
        mpy     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;h21*x1
        mac     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;+h22*x2
        macr    y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;+h23*x3
        move              a,x:(r1)+                 ;store y2
        mpy     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;h31*x1
        mac     y0,x0,a   x:(r0)+,y0  x:(r3)+,x0    ;+h32*x2
        macr    y0,x0,a                             ;+h33*x3
        move              a,x:(r1)+		    ;store y3

ASM_APP_2
	END_PROFILING:
ASM_APP_3

	move	x:-(r2),m0		; restore m0 register
	move	x:-(r2),ssh		; restore ssh 
	rts
	endsec

