;
;	annotate version 1.11
;
	section	mat1x3_arc
	opt so,nomd
	page 132,66,3,3

;*** DSP56000/1 Motorola 1.11 GNU 1.37.1

; /*
;  * benchmark program:   mat1x3_arc.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  * 
;  * description:         1x3 matrix - multiply benchmarking
;  *
;  *               |h11 h12 h13|   |x1|   |y1|   | h11*x1+h12*x2+h31*x3 |
;  *               |h21 h22 h23| x |x2| = |y2| = | h21*x1+h22*x2+h23*x3 |
;  *               |h31 h32 h33|   |x3|   |y3|   | h31*x1+h32*x2+h33*x3 |
;  * 
;  * Elements are to store in following order:
;  * 
;  * matrix h[9]={h11,h12,h13, h21,h22,h23, h31,h32,h33}
;  * vector x[3]={x1,x2,x3}
;  * vector y[3]={y1,y1,y3}
;  * 
;  * reference code:   Motorola's Dr.Bub/56100/matrix/matmul1.asm (modified)
;  * 
;  * f. verification:      with printf function
;  * 
;  * organization:         Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             06-04-94 creation (Martinez Velarde)
;  * 
;  */

	org	x:
F___x
	dc 2	;x1
	dc -2	;x2
	dc 9	;x3

	org	y:
F___h
	dc 1	;h11
	dc 1	;h12
	dc 1	;h13
	dc 2	;h21
	dc 2	;h22
	dc 2	;h23
	dc 3	;h31
	dc 3	;h32
	dc 3	;h33
F___y
	ds 3	; save 3 spaces for output vector

LC0	dc	'matrix element',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain	
	move	ssh,y:(r6)+
	move	m0,y:(r6)+

        move    #F___x,r0       ;point to X vector
        move    #F___h,r4       ;point to H matrix
        move    #2,m0           ;mod 3
        move    #F___y,r1       ;output Y vector
	nop

ASM_APP_0
	START_PROFILING:
ASM_APP_1

        move             x:(r0)+,x0  y:(r4)+,y0

	do	#3,ENDLOOP
        mpy     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
        mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
        mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
	asr	a
        move    a0,y:(r1)+
ENDLOOP

ASM_APP_2
	END_PROFILING:
ASM_APP_3

	move	y:-(r6),m0

; print output value
;

	move	#LC0,r1
        move    #F___y,r3       ;output Y vector
	do	#3,L1
	move	y:(r3)+,a0
	move	a0,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-
L1
	move	y:-(r6),ssh
	rts
	endsec


