;/*
; * benchmark program:   n_complex_update_arc.asm
; * 
; * benchmark suite:     DSP-kernel
; * 
; * description:         n complex updates - filter benchmarking
; * 
; * This program performs n complex updates of the form 
; *           D(i) = C(i) + A(i)*B(i),
; * where A(i), B(i), C(i) and D(i) are complex numbers,
; * and i = 1,...,N
; * 
; *          A(i) = Ar(i) + j Ai(i)
; *          B(i) = Br(i) + j Bi(i)
; *          C(i) = Cr(i) + j Ci(i)
; *          D(i) = C(i) + A(i)*B(i) =   Dr(i)  +  j Di(i)
; *                      =>  Dr(i) = Cr(i) + Ar(i)*Br(i) - Ai(i)*Bi(i)
; *                      =>  Di(i) = Ci(i) + Ar(i)*Bi(i) + Ai(i)*Br(i); * 
; * 
; * reference code:      target assembly
; * 
; * f. verification:     simulator based
; * 
; *  organization:        Aachen University of Technology - IS2 
; *                       DSP Tools Group
; *                       phone:  +49(241)807887 
; *                       fax:    +49(241)8888195
; *                       e-mail: zivojnov@ert.rwth-aachen.de 
; *
; * author:              Juan Martinez Velarde
; * 
; * history:             11-5-94 creation (Martinez Velarde)
; * 
; */
	section n_complex_updates_arc
	opt	so,nomd,cc

        org     x:$0
        org     x:$0
V__A:   dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1
	dc      2
	dc      1

V__C:   dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4
	dc      3
	dc	4

        org     y:$0
V__B:   dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5
	dc      2
	dc      5

V__D:   dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0
	dc      0

LC0	dc	'dr',$3D,$25,'i',$0A,$0A,$00
LC1	dc	'di',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,y:(r6)+

        move 	#V__A,r3
	move	#V__D,r2
        move 	#V__C,r1
	move	#V__B,r4
	clr	a
	clr	b

	START_PROFILING:

	do	#16,END_PROFILING

        move             x:(r1)+,a
	move	a,a0	
	asl	a	 x:(r1)+,b
	move	b,b0                    
	asl	b        x:(r3)+,x1  y:(r4)+,y1   ;x1=ar,y1=br
        mac     y1,x1,a  x:(r3)+,x0  y:(r4)+,y0   ;a+=ar*br,x0=ai,y0=bi
        mac    -y0,x0,a                           ;a-=ai*bi
	mac	y1,x0,b                           ;b+=ar*bi
	mac     y0,x1,b                           ;b+=ai*br
	asr 	a 
	asr 	b	a0,y:(r2)+                ;store a
       	move            b0,y:(r2)+                ;store b

	END_PROFILING:

; **************************************************
	
	move	#V__D,r2

	do	#16,ENDLOOP

	move	#LC0,r1
	clr	a
	move	y:(r2)+,a0
	move	a0,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-

	move	#LC1,r1
	clr	b
	move	y:(r2)+,b0
	move	b0,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-

ENDLOOP:

; **************************************************

	move	y:-(r6),ssh
	rts
	endsec