/*
 * benchmark program:   n_real_updates_ext.c
 * 
 * benchmark suite:     DSP-kernel
 * 
 * description:         n_real_updates - filter benchmarking
 * 
 * This program performs n real updates of the form 
 *           D(i) = C(i) + A(i)*B(i),
 * where A(i), B(i), C(i) and D(i) are real numbers,
 * and i = 1,...,N
 * 
 * reference code:      target assembly
 * 
 * f. verification:     simulator based
 * 
 *  organization:        Aachen University of Technology - IS2 
 *                       DSP Tools Group
 *                       phone:  +49(241)807887 
 *                       fax:    +49(241)8888195
 *                       e-mail: zivojnov@ert.rwth-aachen.de 
 *
 * author:              Juan Martinez Velarde
 * 
 * history:             25-05-94 creation (Martinez Velarde)
 *                      10-03-95 language extensions (Harald L. Schraut)
 *
 *                      $Author: schraut $
 *                      $Date: 1995/03/10 10:59:30 $
 *                      $Revision: 1.1 $
 */

#define STORAGE_CLASS register
#define TYPE          int
#define N             16

void
pin_down(TYPE pm *pa, TYPE dm *pb, TYPE dm *pc, TYPE dm *pd)
{
  STORAGE_CLASS int i ; 

  for (i=0 ; i < N ; i++)
    {
      *pa++ = 10 ; 
      *pb++ = 2 ; 
      *pc++ = 10 ; 
      *pd++ = 0 ; 
    }
}

TYPE main()
{
  static TYPE pm A[N];
  static TYPE dm B[N], C[N], D[N] ; 
  
  STORAGE_CLASS TYPE pm *p_a = &A[0];
  STORAGE_CLASS TYPE dm *p_b = &B[0];
  STORAGE_CLASS TYPE dm *p_c = &C[0], *p_d = &D[0] ;
  STORAGE_CLASS TYPE i ; 

  pin_down(&A[0], &B[0], &C[0], &D[0]) ; 
  
  START_PROFILING; 
	  
  for (i = 0 ; i < N ; i++) 
    *p_d++  = *p_c++ + *p_a++ * *p_b++ ;
  
  END_PROFILING;   
  
  pin_down(&A[0], &B[0], &C[0], &D[0]) ; 
  return(0)  ; 
}
